@@ -40,8 +40,9 @@ do_unmount()
 
 		retry=4 # Effectively TERM, sleep 1, TERM, sleep 1, KILL, sleep 1
 		while ! LC_ALL=C $cmd "$mnt" 2>/dev/null; do
-			if type fuser >/dev/null 2>&1; then
-				pids="$(fuser $f_opts "$mnt" 2>/dev/null)"
+			if command -v fuser >/dev/null 2>&1; then
+				pids="$(timeout -k 10 -s KILL "${rc_fuser_timeout:-60}" \
+					fuser $f_opts "$mnt" 2>/dev/null)"
 			fi
 			case " $pids " in
 				*" $$ "*)
@@ -59,7 +60,7 @@ do_unmount()
 						eend 1
 					else
 						local sig="TERM"
-						retry=$(($retry - 1))
+						: $(( retry -= 1 ))
 						[ $retry = 1 ] && sig="KILL"
 						fuser $f_kill$sig -k $f_opts \
 							"$mnt" >/dev/null 2>&1
