@@ -74,18 +74,10 @@ static struct pam_conv conv = { NULL, NULL};
 
 #include "builtins.h"
 #include "einfo.h"
+#include "queue.h"
 #include "rc.h"
 #include "rc-misc.h"
 
-/* Some libc implementations don't define this */
-#ifndef LIST_FOREACH_SAFE
-#define	LIST_FOREACH_SAFE(var, head, field, tvar)			      \
-	for ((var) = LIST_FIRST((head));				      \
-	     (var) && ((tvar) = LIST_NEXT((var), field), 1);		      \
-	     (var) = (tvar))
-#endif
-
-
 typedef struct scheduleitem
 {
 	enum
@@ -307,7 +299,7 @@ parse_schedule(const char *string, int timeout)
 }
 
 static pid_t
-get_pid(const char *pidfile, bool quiet)
+get_pid(const char *pidfile)
 {
 	FILE *fp;
 	pid_t pid;
@@ -334,8 +326,7 @@ get_pid(const char *pidfile, bool quiet)
 /* return number of processed killed, -1 on error */
 static int
 do_stop(const char *exec, const char *const *argv,
-    pid_t pid, uid_t uid,int sig,
-    bool quiet, bool verbose, bool test)
+    pid_t pid, uid_t uid,int sig, bool test)
 {
 	RC_PIDLIST *pids;
 	RC_PID *pi;
@@ -353,20 +344,16 @@ do_stop(const char *exec, const char *const *argv,
 
 	LIST_FOREACH_SAFE(pi, pids, entries, np) {
 		if (test) {
-			if (!quiet)
-				einfo("Would send signal %d to PID %d", sig, pi->pid);
+			einfo("Would send signal %d to PID %d", sig, pi->pid);
 			nkilled++;
 		} else {
-			if (verbose)
-				ebegin("Sending signal %d to PID %d",
-				    sig, pi->pid);
+			ebeginv("Sending signal %d to PID %d", sig, pi->pid);
 			errno = 0;
 			killed = (kill(pi->pid, sig) == 0 ||
 			    errno == ESRCH ? true : false);
-			if (verbose)
-				eend(killed ? 0 : 1,
-				    "%s: failed to send signal %d to PID %d: %s",
-				    applet, sig, pi->pid, strerror(errno));
+			eendv(killed ? 0 : 1,
+				"%s: failed to send signal %d to PID %d: %s",
+				applet, sig, pi->pid, strerror(errno));
 			if (!killed) {
 				nkilled = -1;
 			} else {
@@ -384,7 +371,7 @@ do_stop(const char *exec, const char *const *argv,
 static int
 run_stop_schedule(const char *exec, const char *const *argv,
     const char *pidfile, uid_t uid,
-    bool quiet, bool verbose, bool test, bool progress)
+    bool test, bool progress)
 {
 	SCHEDULEITEM *item = TAILQ_FIRST(&schedule);
 	int nkilled = 0;
@@ -396,15 +383,15 @@ run_stop_schedule(const char *exec, const char *const *argv,
 	const char *const *p;
 	bool progressed = false;
 
-	if (verbose) {
-		if (exec)
-			einfo ("Will stop %s", exec);
-		if (pidfile)
-			einfo("Will stop PID in pidfile `%s'", pidfile);
-		if (uid)
-			einfo("Will stop processes owned by UID %d", uid);
-		if (argv && *argv) {
-			einfon("Will stop processes of `");
+	if (exec)
+		einfov("Will stop %s", exec);
+	if (pidfile)
+		einfov("Will stop PID in pidfile `%s'", pidfile);
+	if (uid)
+		einfov("Will stop processes owned by UID %d", uid);
+	if (argv && *argv) {
+		einfovn("Will stop processes of `");
+		if (rc_yesno(getenv("EINFO_VERBOSE"))) {
 			for (p = argv; p && *p; p++) {
 				if (p != argv)
 					printf(" ");
@@ -415,7 +402,7 @@ run_stop_schedule(const char *exec, const char *const *argv,
 	}
 
 	if (pidfile) {
-		pid = get_pid(pidfile, false);
+		pid = get_pid(pidfile);
 		if (pid == -1)
 			return 0;
 	}
@@ -428,8 +415,7 @@ run_stop_schedule(const char *exec, const char *const *argv,
 
 		case SC_SIGNAL:
 			nrunning = 0;
-			nkilled = do_stop(exec, argv, pid, uid, item->value,
-			    quiet, verbose, test);
+			nkilled = do_stop(exec, argv, pid, uid, item->value, test);
 			if (nkilled == 0) {
 				if (tkilled == 0) {
 					if (progressed)
@@ -458,8 +444,7 @@ run_stop_schedule(const char *exec, const char *const *argv,
 				     nloops++)
 				{
 					if ((nrunning = do_stop(exec, argv,
-						    pid, uid, 0, true, false,
-	test)) == 0)
+						    pid, uid, 0, test)) == 0)
 						return 0;
 
 
@@ -676,8 +661,6 @@ start_stop_daemon(int argc, char **argv)
 	bool stop = false;
 	bool oknodo = false;
 	bool test = false;
-	bool quiet;
-	bool verbose = false;
 	char *exec = NULL;
 	char *startas = NULL;
 	char *name = NULL;
@@ -914,8 +897,6 @@ start_stop_daemon(int argc, char **argv)
 	endpwent();
 	argc -= optind;
 	argv += optind;
-	quiet = rc_yesno(getenv("EINFO_QUIET"));
-	verbose = rc_yesno(getenv("EINFO_VERBOSE"));
 
 	/* Allow start-stop-daemon --signal HUP --exec /usr/sbin/dnsmasq
 	 * instead of forcing --stop --oknodo as well */
@@ -1059,7 +1040,7 @@ start_stop_daemon(int argc, char **argv)
 		else
 			parse_schedule(NULL, sig);
 		i = run_stop_schedule(exec, (const char *const *)margv,
-		    pidfile, uid, quiet, verbose, test, progress);
+		    pidfile, uid, test, progress);
 
 		if (i < 0)
 			/* We failed to stop something */
@@ -1081,16 +1062,16 @@ start_stop_daemon(int argc, char **argv)
 	}
 
 	if (pidfile)
-		pid = get_pid(pidfile, true);
+		pid = get_pid(pidfile);
 	else
 		pid = 0;
 
 	if (do_stop(exec, (const char * const *)margv, pid, uid,
-		0, true, false, test) > 0)
+		0, test) > 0)
 		eerrorx("%s: %s is already running", applet, exec);
 
 	if (test) {
-		if (quiet)
+		if (rc_yesno(getenv("EINFO_QUIET")))
 			exit (EXIT_SUCCESS);
 
 		einfon("Would start");
@@ -1114,10 +1095,8 @@ start_stop_daemon(int argc, char **argv)
 		exit(EXIT_SUCCESS);
 	}
 
-	if (verbose) {
-		ebegin("Detaching to start `%s'", exec);
-		eindent();
-	}
+	ebeginv("Detaching to start `%s'", exec);
+	eindentv();
 
 	/* Remove existing pidfile */
 	if (pidfile)
@@ -1285,9 +1264,9 @@ start_stop_daemon(int argc, char **argv)
 		}
 
 		/* We don't redirect stdin as some daemons may need it */
-		if (background || quiet || redirect_stdout)
+		if (background || redirect_stdout || rc_yesno(getenv("EINFO_QUIET")))
 			dup2(stdout_fd, STDOUT_FILENO);
-		if (background || redirect_stderr)
+		if (background || redirect_stderr || rc_yesno(getenv("EINFO_QUIET")))
 			dup2(stderr_fd, STDERR_FILENO);
 
 		for (i = getdtablesize() - 1; i >= 3; --i)
@@ -1318,9 +1297,10 @@ start_stop_daemon(int argc, char **argv)
 				return -1;
 			}
 		} while (!WIFEXITED(i) && !WIFSIGNALED(i));
-		if (!WIFEXITED(i) || WEXITSTATUS(i) != 0)
-			eerrorx("%s: failed to start `%s'", applet, exec);
-
+		if (!WIFEXITED(i) || WEXITSTATUS(i) != 0) {
+			eerror("%s: failed to start `%s'", applet, exec);
+			exit(EXIT_FAILURE);
+		}
 		pid = spid;
 	}
 
@@ -1354,7 +1334,7 @@ start_stop_daemon(int argc, char **argv)
 				alive = true;
 		} else {
 			if (pidfile) {
-				pid = get_pid(pidfile, false);
+				pid = get_pid(pidfile);
 				if (pid == -1) {
 					eerrorx("%s: did not "
 					    "create a valid"
@@ -1364,7 +1344,7 @@ start_stop_daemon(int argc, char **argv)
 			} else
 				pid = 0;
 			if (do_stop(exec, (const char *const *)margv,
-				pid, uid, 0, true, false, test) > 0)
+				pid, uid, 0, test) > 0)
 				alive = true;
 		}
 
